'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createMigration;

var _constants = require('redux-persist/constants');

function createMigration(manifest, versionSelector, versionSetter) {
  if (typeof versionSelector === 'string') {
    (function () {
      var versionString = versionSelector;
      versionSelector = function versionSelector(state) {
        return state && state[versionString] && state[versionString].version;
      };
      versionSetter = function versionSetter(state, version) {
        if (state && state[versionString]) state[versionString].version = version;
        return state;
      };
    })();
  }

  var versionKeys = Object.keys(manifest).sort();
  var currentVersion = versionKeys[versionKeys.length - 1];

  return function (next) {
    return function (reducer, initialState, enhancer) {
      return next(createMigrationReducer(reducer), initialState, enhancer);
    };
  };

  function createMigrationReducer(reducer) {
    return function (state, action) {
      if (action.type !== _constants.REHYDRATE) return reducer(state, action);else {
        var incomingState = action.payload;
        var incomingVersion = versionSelector(incomingState);
        if (incomingVersion === currentVersion) return reducer(state, action);

        var migratedState = migrate(incomingState, incomingVersion);
        action.payload = migratedState;
        return reducer(migratedState, action);
      }
    };
  }

  function migrate(state, version) {
    versionKeys.filter(function (v) {
      return !version || v > version;
    }).forEach(function (v) {
      state = manifest[v](state);
    });
    state = versionSetter(state, currentVersion);
    return state;
  }
}